/////////////////////////////////////////////////////////////////////////
// CGI Script to display records from a database
// Written by George Cross, Borland C++ Developer Support
// Halloween 1996
//
// Build this source file using Borland C++ 5.0x as follows:
//
//   bcc32 -c thisfile.cpp
//   tlink32 c0x32 thisfile.obj,,,idapi32.lib import.lib cw32.lib,,
/////////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include <idapi.h>

/////////////////////////////////////////////////////////////////////////
//  Prototypes and Globals
/////////////////////////////////////////////////////////////////////////

const char* gszAliasName = "DSHOURS";
const char* gszGetHourExe = "/cgi-dos/gethour.exe";
const char* gszShowHourExe = "/cgi-dos/showhour.exe";

inline void __fastcall ErrorCheck(DBIResult result);
void QueryDatabase(hDBIDb* phDb, hDBICur* phQryCur);
void DisplayBDEError(DBIResult result);
void PrepareToReadHoursDatabase(hDBIDb* phDb, hDBICur* phCur,
                                char** ppszRecBuf, FLDDesc** ppfielddesc,
                                CURProps* pcurProps);
void CloseHoursDatabase(hDBIDb* phDb, hDBICur* phCur,
                        char* pszRecBuf, FLDDesc* pfielddesc);
void PrintForm();
void PrintRecord(hDBICur* phCur, char* pRecBuf);
void PrintTable(hDBICur* phCur, char* pRecBuf);
void QueryDatabase(hDBIDb* phDb, hDBICur* phQryCur);

/////////////////////////////////////////////////////////////////////////
//  Utility functions for calling IDAPI
/////////////////////////////////////////////////////////////////////////

//This inline function is used to throw exceptions if an IDAPI function
//fails.
inline void __fastcall ErrorCheck(DBIResult result)
{
  if (result != DBIERR_NONE) throw result;
}

//This function simply displays a message box notifying the user of an
//IDAPI function failure.
void DisplayBDEError(DBIResult result)
{
  char errstr[DBIMAXMSGLEN+1];
  DbiGetErrorString(result, errstr);
  cout << "IDAPI Error: " << errstr << endl;
}

#pragma option -w-aus -w-par
//This function just opens the 'HOURS' database and prepares to read it.
void PrepareToReadHoursDatabase(hDBIDb* phDb, hDBICur* phCur,
                                char** ppszRecBuf, FLDDesc** ppfielddesc,
                                CURProps* pcurProps)
{
  DBIResult result;
  DbiInit(NULL);
  result = DbiOpenDatabase((char*) gszAliasName,
                           NULL,
                           dbiREADWRITE,
                           dbiOPENSHARED,
                           NULL,
                           0,
                           NULL,
                           NULL,
                           phDb);
  ErrorCheck(result);
}

//This function opens the table requested and allocates the record and
//field descriptors.
void OpenTable(hDBIDb* phDb, hDBICur* phCur,
               char** ppszRecBuf, FLDDesc** ppfielddesc,
               CURProps* pcurProps, char* pszTable)
{
  DBIResult result;
  result = DbiOpenTable(*phDb,
                        pszTable,
                        szPARADOX,
                        NULL,
                        NULL,
                        0,
                        dbiREADONLY,
                        dbiOPENSHARED,
                        xltFIELD,
                        false,
                        NULL,
                        phCur);
  ErrorCheck(result);

  result = DbiGetCursorProps(*phCur, pcurProps);
  ErrorCheck(result);

  *ppszRecBuf = new char[pcurProps->iRecSize];
  *ppfielddesc = new FLDDesc[pcurProps->iFields];

  result = DbiGetFieldDescs(*phCur, *ppfielddesc);
  ErrorCheck(result);
}

#pragma option -w.aus -w.par
//This function just releases those resources allocated for using this
//database, and then closes the database
void CloseHoursDatabase(hDBIDb* phDb, hDBICur* phCur,
                        char* pszRecBuf, FLDDesc* pfielddesc)
{
  if (*phCur)
    DbiCloseCursor(phCur);
  if (pszRecBuf)
    delete[] pszRecBuf;
  if (pfielddesc)
    delete[] pfielddesc;
  if (*phDb)
    DbiCloseDatabase(phDb);
  DbiExit();
}

/////////////////////////////////////////////////////////////////////////
// These are the functions which send HTML to STDOUT.
/////////////////////////////////////////////////////////////////////////

//This function prints the first part of the page
void PrintHeading()
{
  cout << "Content-type: text/html\n\n "
       << "<html>"
       << "<title>DS Extra Departmental Hours</title>" << endl
       << "<body><font size=+2>DS Hours Database<font size=+1><p>" << endl
       << "Select your name and project for which you want<br>"
          "to change your hours."
       << "<form method=\"post\" action=\""
       << gszGetHourExe << "\">" << endl;
}

//This function prints the trailing details on the form
void PrintFooting()
{
  cout << "<input type=\"submit\" value=\"Get Hours\"></form>" << endl
       << "<p><HR width=\"100%\"><p>"
       << "<form method=\"post\" action=\""
       << gszShowHourExe << "\">" << endl
       << "Or, just show the table of all hours." << endl
       << "<input type=\"submit\" value=\"Show All Hours\"></form>" ;
}

//This function prints an individual record
void PrintRecord(hDBICur* phCur, char* pRecBuf)
{
  char buf[50];

  DbiGetField(*phCur, 1, pRecBuf, buf, NULL);
  cout << "<option>" << buf << endl;
}

//This function iterates through all records in the table.
void PrintTable(hDBICur* phCur, char* pRecBuf, FLDDesc* pfielddesc)
{
  char buf[50];

  DbiSetToBegin(*phCur);
  cout << pfielddesc[0].szName << endl
       << "<select name=" << pfielddesc[0].szName << ">" << endl;

  while(DbiGetNextRecord(*phCur, dbiNOLOCK, pRecBuf, NULL) == DBIERR_NONE)
    PrintRecord(phCur, pRecBuf);

  cout << "</select>" << endl;
}

/////////////////////////////////////////////////////////////////////////
// This function is the main entry point for this module
/////////////////////////////////////////////////////////////////////////
int main()
{
  hDBIDb hDb = NULL;               //Handle to the database
  hDBICur hCur = NULL;             //Handle to the 'HOURS' table
  char* pszRecBuf = NULL;          //Buffer to hold an entire record
  FLDDesc* pfielddesc = NULL;      //Buffer to hold field descriptions
  CURProps curProps;               //Structure to hold table properties

  try{
    PrepareToReadHoursDatabase(&hDb,&hCur, &pszRecBuf, &pfielddesc, &curProps);
    PrintHeading();

    OpenTable(&hDb,&hCur, &pszRecBuf, &pfielddesc, &curProps, "ENGINEER");
    PrintTable(&hCur, pszRecBuf, pfielddesc);
    delete[] pszRecBuf;
    delete[] pfielddesc;
    pszRecBuf = NULL;
    pfielddesc = NULL;

    OpenTable(&hDb,&hCur, &pszRecBuf, &pfielddesc, &curProps, "PROJECT");
    PrintTable(&hCur, pszRecBuf, pfielddesc);

    PrintFooting();
    CloseHoursDatabase(&hDb, &hCur, pszRecBuf, pfielddesc);
  }
  catch(DBIResult result){
    DisplayBDEError(result);
    CloseHoursDatabase(&hDb, &hCur, pszRecBuf, pfielddesc);
  }

  return 0;
}


