/////////////////////////////////////////////////////////////////////////
// CGI Script to display records from a database
// Written by George Cross, Borland C++ Developer Support
// November 5 1996
//
// Build this source file using Borland C++ 5.0x as follows:
//
//   bcc32 -c thisfile.cpp
//   tlink32 c0x32 thisfile.obj,,,idapi32.lib import.lib cw32.lib,,
/////////////////////////////////////////////////////////////////////////

#include <stdio.h>              //for sprintf() and fscanf()
#include <stdlib.h>             //for atoi()
#include <iostream.h>           //for cout
#include <except.h>             //for xmsg class
#include <process.h>            //for spawnl()
#include <cstring.h>            //for string class
#include <idapi.h>              //for IDAPI
#pragma option -K
/////////////////////////////////////////////////////////////////////////
//  Prototypes and Globals
/////////////////////////////////////////////////////////////////////////

const char* gszAliasName = "DSHOURS";                  //database name
const char* gszShowHourExe = "/cgi-dos/showhour.exe";  //exe to submit query to
const int MAX_CONTENT_LENGTH = 500;                    //used in GetCGIArgs

inline void __fastcall ErrorCheck(DBIResult result);
void DisplayBDEError(DBIResult result);
void PrepareToWriteHoursDatabase(hDBIDb* phDb);
void CloseHoursDatabase(hDBIDb* phDb, hDBICur* phCur,
                        char* pszRecBuf, FLDDesc* pfielddesc);
void UpdateHoursDatabase(hDBIDb hDb, char* pszEngineerName,
                        char* pszProject, float hours);
void GetCGIArgs(float* hours);
void ConvertCharInString(char* pszString, char oldchar, char newchar);

/////////////////////////////////////////////////////////////////////////
//  Utility functions for calling IDAPI
/////////////////////////////////////////////////////////////////////////

//This inline function is used to throw exceptions if an IDAPI function
//fails.
inline void __fastcall ErrorCheck(DBIResult result)
{
  if (result != DBIERR_NONE) throw result;
}

//This function simply displays a message box notifying the user of an
//IDAPI function failure.
void DisplayBDEError(DBIResult result)
{
  char errstr[DBIMAXMSGLEN+1];
  DbiGetErrorString(result, errstr);
  cout << "IDAPI Error: " << errstr << endl;
}

#pragma option -w-aus -w-par
//This function just opens the 'HOURS' database and prepares to read it.
void PrepareToWriteHoursDatabase(hDBIDb* phDb)
{
  DBIResult result;
  DbiInit(NULL);
  result = DbiOpenDatabase((char*) gszAliasName,
                           NULL,
                           dbiREADWRITE,
                           dbiOPENSHARED,
                           NULL,
                           0,
                           NULL,
                           NULL,
                           phDb);
  ErrorCheck(result);
}

#pragma option -w.aus -w.par
//This function just releases those resources allocated for using this
//database, and then closes the database
void CloseHoursDatabase(hDBIDb* phDb)
{
  if (*phDb)
    DbiCloseDatabase(phDb);
  DbiExit();
}

/////////////////////////////////////////////////////////////////////////
//These functions update the hours for the current record
//I know I use a lot of SQL and that perhaps it is inefficient.  But it
//is real easy, I'm lazy at this point and it is good enough for this project.
/////////////////////////////////////////////////////////////////////////

//This function does some SQL on the database
#pragma warn -aus
void UpdateHoursDatabase(hDBIDb hDb, char* pszEngineerName,
                        char* pszProject, float hours)
{
  DBIResult result;
  hDBICur hCur = NULL;       //Handle to the cursor for the HOURS.DB table
  char* pRecBuf = NULL;      //Buffer for a record in the table
  CURProps curProps;         //Area for the table cursor properties
  char buf[256];

  result = DbiOpenTable(hDb,
                        "hours.DB",
                        szPARADOX,
                        NULL,
                        NULL,
                        0,
                        dbiREADWRITE,
                        dbiOPENSHARED,
                        xltFIELD,
                        FALSE,
                        NULL,
                        &hCur);
  ErrorCheck(result);

  result = DbiGetCursorProps(hCur,&curProps);
  ErrorCheck(result);

  pRecBuf = new char[curProps.iRecSize * sizeof(char)];
  ErrorCheck(result);

  //Fill in record buffer with that on which we want to search
  result = DbiPutField(hCur, 1, pRecBuf, pszEngineerName);
  ErrorCheck(result);
  result = DbiPutField(hCur, 2, pRecBuf, pszProject);
  ErrorCheck(result);

  result = DbiSetToKey(hCur,
                       keySEARCHEQ,
                       FALSE,
                       0,
                       0,
                       pRecBuf);
  delete[] pRecBuf;
  switch (result){
    case DBIERR_NONE:
      sprintf(buf,
              "%s%f%s%s%s%s%s",
              "UPDATE hours.DB SET Hours = ",
              hours,
              " WHERE (Project = '",
              pszProject,
              "') AND (Engineer = '",
              pszEngineerName,
              "')");
      result = DbiQExecDirect(hDb, qrylangSQL, buf, &hCur);
      ErrorCheck(result);
      break;
    case DBIERR_RECNOTFOUND:
      sprintf(buf,
              "%s%s%s%s%s%f%s",
              "INSERT INTO hours.DB (Engineer,Project,Hours) VALUES ('",
              pszEngineerName,
              "','",
              pszProject,
              "',",
              hours,
              ")");

      result = DbiQExecDirect(hDb, qrylangSQL, buf, NULL);
      ErrorCheck(result);
      break;
    default:
      ErrorCheck(result);
  }
}
#pragma warn .aus

/////////////////////////////////////////////////////////////////////////
//These functions get the arguments for us from STDIN
/////////////////////////////////////////////////////////////////////////

// This function is retrieves the engineer's name and the project
// from STDIN supplied by CGI
void GetCGIArgs(float* pdNewHours)
{
  char buf[MAX_CONTENT_LENGTH];
  char* p1;
  int length;
  int i;

  //Determine the length of everything waiting in STDIN.
  //This information is stored in the CONTENT_LENGTH environment
  //variable and must be converted from its string value to an integer
  p1 = getenv("CONTENT_LENGTH");
  if (p1 != NULL)
    length = atoi(p1);
  else{
    sprintf (buf,
             "%s%s%d%s",
             __FILE__,
             " ",
             __LINE__,
             " Error: Could not determine CONTENT_LENGTH.");
    string estr(buf);
    throw xmsg(estr);
  }

  //Read in through everything waiting in STDIN and find the hours
  for (i=0;i<length;i++){
    fscanf(stdin, "%c", &buf[i]);
    if (buf[i] == '=')
      fscanf(stdin, "%f", pdNewHours);
  }
  buf[i] = '\0';

  return;
}

//This function converts any '+' symbols to spaces
void ConvertCharInString(char* pszString, char oldchar, char newchar)
{
  char* p = pszString;
  while (p){
    p = strchr(p, oldchar);
    if (p)
      *p = newchar;
  }
}


/////////////////////////////////////////////////////////////////////////
// This function is the main entry point for this module
// Arguments are:
//   argv[1] - The name of the engineer
//   argv[2] - The project on which the engineer will update the hours worked
/////////////////////////////////////////////////////////////////////////
int main(int /* argc */, char* argv[])
{
  hDBIDb hDb = NULL;               //Handle to the database
  char* szEngineerName = argv[1];  //Buffer to hold the Engineer's Name
  char* szProject = argv[2];       //Buffer to hold the Project Name
  float dNewHours;                //The new value for the hours field

  try{

    //This exe will be passed two arguments, with spaces denoted by
    //a '|' character.  We must convert these '|' characters first.
    ConvertCharInString(szEngineerName, '|', ' ');
    ConvertCharInString(szProject, '|', ' ');

    GetCGIArgs(&dNewHours);

    PrepareToWriteHoursDatabase(&hDb);
    UpdateHoursDatabase(hDb, szEngineerName, szProject, dNewHours);
    CloseHoursDatabase(&hDb);
    cout << "Content-type: text/html\n\n "
         << "<html>"
         << "<title>DS Extra Departmental Hours</title>" << endl
         << "<body><font size=+2>DS Hours Database<font size=+1><p>" << endl
         << "Done ! Click below to show the table of all hours." << endl
         << "<form method=\"post\" action=\""
         << gszShowHourExe << "\">" << endl
         << "<input type=\"submit\" value=\"Show All Hours\"></form>" ;
  }
  catch(DBIResult result){
    DisplayBDEError(result);
    CloseHoursDatabase(&hDb);
  }
  catch(xmsg& x){
    cout << x.why();
  }

  return 0;
}


