/////////////////////////////////////////////////////////////////////////
// CGI Script to display records from a database
// Written by George Cross, Borland C++ Developer Support
// Halloween 1996
//
// Build this source file using Borland C++ 5.0x as follows:
//
//   bcc32 -c thisfile.cpp
//   tlink32 c0x32 thisfile.obj,,,idapi32.lib import.lib cw32.lib,,
/////////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include <idapi.h>

/////////////////////////////////////////////////////////////////////////
//  Prototypes and Globals
/////////////////////////////////////////////////////////////////////////

const char* gszAliasName = "DSHOURS";
inline void __fastcall ErrorCheck(DBIResult result);
void QueryDatabase(hDBIDb* phDb, hDBICur* phQryCur);
void DisplayBDEError(DBIResult result);
void PrepareToReadHoursDatabase(hDBIDb* phDb, hDBICur* phCur,
                                char** ppszRecBuf, FLDDesc** ppfielddesc,
                                CURProps* pcurProps);
void CloseHoursDatabase(hDBIDb* phDb, hDBICur* phCur,
                        char* pszRecBuf, FLDDesc* pfielddesc);
void PrintFieldNames(FLDDesc* pfielddesc);
void PrintRecord(hDBICur* phCur, char* pRecBuf);
void PrintTable(hDBICur* phCur, char* pRecBuf);

/////////////////////////////////////////////////////////////////////////
//  Utility functions for calling IDAPI
/////////////////////////////////////////////////////////////////////////

//This inline function is used to throw exceptions if an IDAPI function
//fails.
inline void __fastcall ErrorCheck(DBIResult result)
{
  if (result != DBIERR_NONE) throw result;
}

//This function simply displays a message box notifying the user of an
//IDAPI function failure.
void DisplayBDEError(DBIResult result)
{
  char errstr[DBIMAXMSGLEN+1];
  DbiGetErrorString(result, errstr);
  cout << "IDAPI Error: " << errstr << endl;
}

#pragma option -w-aus -w-par
//This function just opens the 'HOURS' database and prepares to read it.
void PrepareToReadHoursDatabase(hDBIDb* phDb, hDBICur* phCur,
                                char** ppszRecBuf, FLDDesc** ppfielddesc,
                                CURProps* pcurProps)
{
  DBIResult result;
  DbiInit(NULL);
  result = DbiOpenDatabase((char*) gszAliasName,
                           NULL,
                           dbiREADWRITE,
                           dbiOPENSHARED,
                           NULL,
                           0,
                           NULL,
                           NULL,
                           phDb);
  ErrorCheck(result);

  QueryDatabase(phDb, phCur);

  result = DbiGetCursorProps(*phCur, pcurProps);
  ErrorCheck(result);

  *ppszRecBuf = new char[pcurProps->iRecSize];
  *ppfielddesc = new FLDDesc[pcurProps->iFields];

  result = DbiGetFieldDescs(*phCur, *ppfielddesc);
  ErrorCheck(result);
}
#pragma option -w.aus -w.par

//This function executes an SQL query on the database
void QueryDatabase(hDBIDb* phDb, hDBICur* phQryCur)
{
  char qryBuf[] = {
    "SELECT DISTINCT D.Project, D.Engineer, D.Hours "
    "FROM hours.DB D, project.DB D1 "
    "WHERE "
    "(D1.Project = D.Project) "
    "ORDER BY D.Project, D.Engineer, D.Hours"
  };

  DBIResult result =
    DbiQExecDirect(*phDb, qrylangSQL, qryBuf, phQryCur);
  ErrorCheck(result);
}

//This function just releases those resources allocated for using this
//database, and then closes the database
void CloseHoursDatabase(hDBIDb* phDb, hDBICur* phCur,
                        char* pszRecBuf, FLDDesc* pfielddesc)
{
  if (*phCur)
    DbiCloseCursor(phCur);
  if (pszRecBuf)
    delete[] pszRecBuf;
  if (pfielddesc)
    delete[] pfielddesc;
  if (*phDb)
    DbiCloseDatabase(phDb);
  DbiExit();
}

/////////////////////////////////////////////////////////////////////////
// These are the functions which send HTML to STDOUT.
/////////////////////////////////////////////////////////////////////////

//This function prints the titles of each field
void PrintFieldNames(FLDDesc* pfielddesc)
{
  cout << "Content-type: text/html\n\n "
       << "<html>"
       << "<title>DS Extra Departmental Hours</title>" << endl
       << "<body><font size=+2>DS Hours Database<font size=+1><p>" << endl
       << "<center><table border cellpadding=5><tr>" << endl
       << "<th colspan=2>" << pfielddesc[0].szName << "</th></tr>" << endl
       << "<tr><th>" << pfielddesc[1].szName << "</th>" << endl
       << "<th>" << pfielddesc[2].szName << "</th></tr>"
       << "</tr>" << endl;
}

//This function prints an individual record
void PrintRecord(hDBICur* phCur, char* pRecBuf)
{
  char buffer[50];
  double hours;

  DbiGetField(*phCur, 2, pRecBuf, buffer, NULL);
  cout << "<td>" << buffer << "</td>";
  DbiGetField(*phCur, 3, pRecBuf, (unsigned char*)&hours, NULL);
  cout << "<td>" << hours << "</td>";
}

//This function iterates through all records in the table.
void PrintTable(hDBICur* phCur, char* pRecBuf)
{
  char LastProject[50] = "";
  char CurrentProject[50];

  DbiSetToBegin(*phCur);

  while(DbiGetNextRecord(*phCur, dbiNOLOCK, pRecBuf, NULL) == DBIERR_NONE) {
    DbiGetField(*phCur, 1, pRecBuf, CurrentProject, NULL);
    if (strcmp(LastProject, CurrentProject) != 0){
      strcpy(LastProject, CurrentProject);
      cout << "<tr><th colspan=2></th></tr><tr><th colspan=2>"
           << CurrentProject << "</th></tr>" << endl;
    }
    cout << "<tr>";
    PrintRecord(phCur, pRecBuf);
    cout << "</tr>" << endl;
  }
  cout << "</table>" << endl;
}

/////////////////////////////////////////////////////////////////////////
// This function is the main entry point for this module
/////////////////////////////////////////////////////////////////////////
int main()
{
  hDBIDb hDb = NULL;               //Handle to the database
  hDBICur hCur = NULL;             //Handle to the 'HOURS' table
  char* pszRecBuf = NULL;          //Buffer to hold an entire record
  FLDDesc* pfielddesc = NULL;      //Buffer to hold field descriptions
  CURProps curProps;               //Structure to hold table properties

  try{
    PrepareToReadHoursDatabase(&hDb,&hCur, &pszRecBuf, &pfielddesc, &curProps);
    PrintFieldNames(pfielddesc);
    PrintTable(&hCur, pszRecBuf);
    CloseHoursDatabase(&hDb, &hCur, pszRecBuf, pfielddesc);
  }
  catch(DBIResult result){
    DisplayBDEError(result);
    CloseHoursDatabase(&hDb, &hCur, pszRecBuf, pfielddesc);
  }

  return 0;
}


